# Python 3 + pySerial による
# 中央精機製ステージコントローラ QT-A / QT-B と通信するためのサンプルです。
# サンプルのため最低限の記述を心がけています。

# Python の記述方法、エラー処理、実装方法など開発言語についてのお問い合わせはお受けできません。
# お使いの開発環境のヘルプなどをご参照いただくか、Python 公式サイトのドキュメントをご確認ください。
#
# Python 公式サイト: https://www.python.org/
# Python.jp プログラミング言語 Python 情報サイト: https://www.python.jp
# pySerial: https://pyserial.readthedocs.io/en/latest/index.html

#
# 【動作確認環境】
# ・ OS: Windows 10 22H2 / Windows 11 23H2
# ・ Python 3.12.3
# ・ pySerial 3.5
# 
# 【前提条件】
# ・ このサンプルではコントローラの A 軸を動作させます。
# ・ コントローラとステージが正しく接続され、正常に動作する設定になっていることとします。
# ・ コントローラはレスポンスなし (X:0) を想定しています。
# ・ 使用する Python に pySerial がインストールされているものとします。
#    pySerial が未インストールの場合、インストールしてください。
#    詳細: https://pyserial.readthedocs.io/en/latest/pyserial.html#installation
#    - PyPI からインストールする場合
#      python -m pip install pyserial
#    - Conda からインストールする場合
#      conda install pyserial
#      もしくは
#      conda install -c conda-forge pyserial
#
# 【サンプルコードの内容】
# サンプルコードは次のようになっています。
# 1. pySerial をインポートする
#    ソースコードに次の行を記述します。
#    import serial
# 2. 通信ポートのオープンと設定
# 4. QT-A / QT-B との RS-232 通信を行う
#    - A 軸に接続されたステージを +1000 パルス移動開始します。
#    - 0.1 s 待ちます。
#    - Q コマンドを送信し、返答された内容をコンソールに出力します。
# 5. 通信ポートを閉じる
#
# 【使用方法】
# 通信ポートの設定に合わせ、定数を設定します。
# 「設定箇所 - ここから」～「設定箇所 - ここまで」の間をコントローラに合わせ変更します。

import serial
# 次のインポートは time.sleep() を使うためのもので、RS-232C 通信とは関係ありません。
import time

# <<<<< 設定箇所 - ここから >>>>>

# 通信ポートの番号を指定します。
# 通信ポート番号はデバイスマネージャなどで確認します。
PORTNAME = 'COM3'

# 通信ポートのボーレート 4800, 9600, 19200, (QT-B のみ) 38400 のいずれかを指定します。
# お使いのコントローラの設定 (パラメータ No.53) に合わせてください。
BAUDRATE = 9600
# BAUDRATE = 4800
# BAUDRATE = 19200
# BAUDRATE = 38400   # QT-B のみ

# 通信ポートのデータビット長 8 か 7 bit を指定する。
# お使いのコントローラの設定 (パラメータ No.53) に合わせてください。
DATABITS = serial.EIGHTBITS
# DATABITS = serial.SEVENBITS

# 通信ポートのパリティ serial.PARITY_NONE (パリティなし),　serial.PARITY_ODD (奇数パリティ), serial.PARITY_EVEN (偶数パリティ) の何れかを指定する。
# お使いのコントローラの設定 (パラメータ No.53) に合わせてください。
PARITY = serial.PARITY_NONE
# PARITY = serial.PARITY_ODD
# PARITY = serial.PARITY_EVEN

# 信ポートのストップビット長 serial.STOPBITS_ONE, serial.STOPBITS_TWO の何れかを指定する。
# お使いのコントローラの設定 (パラメータ No.53) に合わせてください。
STOPBITS = serial.STOPBITS_ONE
# STOPBITS = serial.STOPBITS_TWO

# コントローラとの通信で使用するデリミタを設定します。
# お使いのコントローラの設定 (パラメータ No.51) に合わせてください。
# ** ℹ 重要 ** #
# pySerial に用意されている Serial.readline()、Serial.writeline() ではコントローラのデリミタが LF である必要があります。
# これらの関数を使用する場合、コントローラのデリミタ設定 (パラメータ No.51) を LF のみにしてください。
DELIMITER = b'\r\n' # CR + LF
#DELIMITER = b'\r' # CR のみ
# DELIMITER = b'\n' # LF のみ

# <<<<< 設定箇所 - ここまで >>>>>


# シリアル通信ポートを開きます。
port = serial.Serial(PORTNAME, BAUDRATE,
                    bytesize=DATABITS,
                    parity=PARITY,
                    stopbits=STOPBITS,
                    timeout=1)

# MGO コマンドを送信します。
# write() 関数に渡すコマンド文字列にはデリミタを付加します。
# write() に渡すコマンドはバイト列にします。
cmd = b'MGO:A+1000'
port.write(cmd+DELIMITER)
print('通信ポート ' + PORTNAME + 'にコマンド' + cmd.decode() + 'を送信しました。')

# この部分は移動がある程度進むのを待つためのものなのでコントローラとの通信とは関係ありません。
time.sleep(0.1)

# Q コマンドを送信し、返答を受信します。
cmd = b'Q:A0'
port.write(cmd+DELIMITER)
print('通信ポート ' + PORTNAME + 'にコマンド' + cmd.decode() + 'を送信しました。')
readed = port.readline().decode().rstrip(DELIMITER.decode())
print('通信ポート ' + PORTNAME + 'からデータ' + readed + 'を受信しました。')

# この部分は受信した文字の処理なのでコントローラとの通信には関係ありません。
pos = readed[:9]
stat = readed[9]
print("A 軸: " + pos + " + " + stat)

# シリアル通信ポートを閉じる
port.close()
